-----------------------------------------------------------
--|> CORE
-----------------------------------------------------------
local ADDON_NAME, NS = ...

--|> Upvalue Globals
-----------------------------------------------------------
local MAX_PLAYER_LEVEL_TABLE = MAX_PLAYER_LEVEL_TABLE
local GetUnitName = GetUnitName
local GetRealmName = GetRealmName
local GetExpansionLevel = GetExpansionLevel
local UnitFactionGroup = UnitFactionGroup
local GetScreenWidth = GetScreenWidth
local GetScreenHeight = GetScreenHeight
local GetFramerate = GetFramerate
local GetCVar = GetCVar
local select = select
local wipe = wipe
local weizPVP = _G.weizPVP

--|> Addon Global Controls
-----------------------------------------------------------
---
--> Enable
function weizPVP:OnEnable()
    NS.EnableEvents() -- Enable Events
    self:CancelAllTimers() -- Pulse Timer
    self:ScheduleRepeatingTimer(NS.PulseEvent, 1)
end

--> Disable
function weizPVP:OnDisable()
    weizPVP_CoreFrame:Hide()
    self:CancelAllTimers() -- Cancel Timers
    NS.ClearListData() -- Clear current player data
    NS.DisableEvents() -- Enable Events
end

--|> Global Functions
-----------------------------------------------------------
--> Clear List Data
function weizPVP.ClearListData()
    NS.ClearListData()
end

--> Toggle Window
function weizPVP.ToggleWindows()
    NS.Options.Window.Visible = not NS.Options.Window.Visible
    NS.SetWindowSettings()
end

--> Toggle Crosshair
function weizPVP.ToggleCrosshair()
    NS.Crosshair.Toggle()
end

--|> Addon Functions
-----------------------------------------------------------
--> Get Player Info
function NS.GetPlayerInfo()
    --> Player Info
    NS.Player = {}
    NS.Player.GUID = UnitGUID("player")
    NS.Player.Name = GetUnitName("player")
    NS.Player.Faction = select(1, UnitFactionGroup("player"))
    NS.Player.Level = UnitLevel("player")
    NS.Player.FromRealm = GetRealmName()
    NS.Player.FromSubRealm = gsub(NS.Player.FromRealm, "[%s%-]", "")
    NS.PlayerRealm = NS.Player.FromSubRealm
end

--> Get Game Info
function NS.GetGameInfo()
    --> Game Info
    NS.Game = {}
    NS.Game.MaxPlayerLevel = MAX_PLAYER_LEVEL_TABLE[GetExpansionLevel()]
end

--> Get Display Info
function NS.GetDisplayInfo()
    NS.Display = {}
    NS.Display.Width = GetScreenWidth()
    NS.Display.Height = GetScreenHeight()
    NS.Display.xFactor = (768 / NS.Display.Width) * (NS.Display.Width / NS.Display.Height)
    NS.Display.yFactor = 768 / NS.Display.Height
    NS.Display.EffectiveScale = UIParent:GetEffectiveScale()
    NS.Display.CVarUiScale = GetCVar("uiScale")
    NS.Display.CurrentFPS = GetFramerate()
end

--> Load Database
function NS.LoadDB()
    -- LOAD CHARACTER OPTIONS
    NS.charDB = LibStub("AceDB-3.0"):New("_weizpvp_chardb", NS._DefaultOptions, true)
    NS.Options = NS.charDB.profile.Options
    NS.KosList = NS.charDB.profile.KosList or {}
    -- LOAD GLOBAL PLAYER DB
    NS.PlayerDB = {}
    NS.globalDB = LibStub("AceDB-3.0"):New("_weizpvp_globaldb", {}, false)
    NS.PlayerDB = NS.globalDB.global
    NS.VersionUpgradeCheck() -- update check
    -- DEBUG CHECK
    if NS.Options.DEBUG then
        weizPVP.NS = NS
    elseif weizPVP.NS then
        wipe(weizPVP.NS)
    end
end

--> Print Addon Message
function NS.PrintAddonMessage(msg)
    -- Print message with default addon prefix
    (SELECTED_CHAT_FRAME or DEFAULT_CHAT_FRAME):AddMessage("|TInterface\\Addons\\weizPVP\\Media\\weizpvp_chat.tga:0|t " .. msg)
end

--> Update Addon Info
function NS.UpdateAddonInfo()
    weizPVP.Database_Version = 3
    weizPVP.Addon_Version = GetAddOnMetadata(ADDON_NAME, "Version")
end

--> Bindings
function NS.SetupBindings()
    _G.BINDING_HEADER_WEIZPVP = NS.Constants.AddonString
    _G.BINDING_NAME_WEIZPVPTOGGLE = "Toggle Window Visibility"
    _G.BINDING_NAME_WEIZPVPCLEARPLAYERLIST = "Clear Player List"
    _G.BINDING_NAME_WEIZPVPTOGGLECROSSHAIR = "Toggle Crosshair"
end

--> Reset All Customization
function NS.ResetAllCustomizations()
    NS.Options.Frames.Header = CopyTable(NS._DefaultOptions.profile.Options.Frames.Header)
    NS.Options.Frames.StatusPopUp = CopyTable(NS._DefaultOptions.profile.Options.Frames.StatusPopUp)
    NS.Options.Bars = CopyTable(NS._DefaultOptions.profile.Options.Bars)
    NS.Options.Frames.Scale = NS._DefaultOptions.profile.Options.Frames.Scale
    NS.Options.Frames.AutoScaleEnabled = NS._DefaultOptions.profile.Options.Frames.AutoScaleEnabled
    NS.Options.Frames.AutoScaleMultiplier = NS._DefaultOptions.profile.Options.Frames.AutoScaleMultiplier
    NS.Options.Frames.BackgroundColor = NS._DefaultOptions.profile.Options.Frames.BackgroundColor
    NS.CoreUI.Initialize()
end
