-----------------------------------------------------------
--|> CoreFrame
--: Creates and updates the addon frames and widgets
-----------------------------------------------------------
--|> Upvalue Globals
-----------------------------------------------------------
local _, NS = ...
local unpack = unpack
local pairs = pairs
local floor = floor
local Round = Round
local InCombatLockdown = InCombatLockdown
local C_Timer_After = C_Timer.After

--> Core UI Table Init
NS.CoreUI = NS.CoreUI or {}
local CUI = NS.CoreUI

--|> LIBS
-----------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

--|> String Constants
-----------------------------------------------------------
local sc = {}
local defaultBackdrop = {}
local function localInit()
    sc = {
        white8x8 = "Interface/BUTTONS/WHITE8X8",
        pinned = "|cFF" .. NS.ColorsLUT["Pinned"].hex .. "PINNED|r",
        unpinned = "|cFF" .. NS.ColorsLUT["Unpinned"].hex .. "UNPINNED|r",
        locked = "|cFF" .. NS.ColorsLUT["Locked"].hex .. "LOCKED|r",
        unlocked = "|cFF" .. NS.ColorsLUT["Unlocked"].hex .. "UNLOCKED|r"
    }

    defaultBackdrop = {
        bgFile = sc.white8x8,
        edgeFile = sc.white8x8,
        edgeSize = NS.Options.Frames.BorderSize
    }
end

--|> Local functions
-----------------------------------------------------------
--> Show Header Tooltip
local pinStat = ""
local lockStat
local windowStatus = ""
local inCombat = ""
local joinString = "|cffbbbbbb - |r"
local function showHeaderToolTip()
    pinStat = ""
    lockStat = ""
    windowStatus = ""
    inCombat = ""
    if NS.Options.Window.Locked == true then
        lockStat = "Unlock"
        windowStatus = NS.Constants.LockIcon
    else
        lockStat = "Lock"
    end

    if NS.Options.Window.Pinned == true then
        pinStat = "Unpin"
        windowStatus = NS.Constants.PinIcon
    else
        pinStat = "Pin"
    end

    if InCombatLockdown() then
        windowStatus = "" .. windowStatus
        inCombat = NS.Constants.InCombat
    end
    --: Build Tooltip
    GameTooltip:SetOwner(weizPVP_CoreBar, "ANCHOR_TOPLEFT")
    GameTooltip:AddDoubleLine(windowStatus .. " " .. NS.Constants.AddonString, inCombat)
    if NS.Options.Window.Pinned == true then
        GameTooltip:AddLine(NS.ColorsLUT["uiMouse"].sStart .. "Right-Click|r" .. joinString .. pinStat)
    elseif NS.Options.Window.Locked == true then
        GameTooltip:AddLine(NS.ColorsLUT["uiMouse"].sStart .. "Ctrl + Right-Click|r" .. joinString .. lockStat)
    else
        GameTooltip:AddLine(NS.ColorsLUT["uiMouse"].sStart .. "Right-Click|r" .. joinString .. pinStat)
        GameTooltip:AddLine(NS.ColorsLUT["uiMouse"].sStart .. "Ctrl + Right-Click|r" .. joinString .. lockStat)
    end
    GameTooltip:Show()
end

--> Adjust Scroll Offset
local function AdustScrollOffset(value)
    if not InCombatLockdown() then
        local vertScroll = weizPVP_CoreFrame.ScrollFrame:GetVerticalScroll()
        local rowHeight = NS.CoreUI.Bar[1].Button:GetHeight()
        local offsetMax = weizPVP_CoreFrame.ScrollFrame.ListFrame:GetHeight() - weizPVP_CoreFrame:GetHeight()
        local numPlayersOnBars = 0
        for _ in pairs(NS.PlayersOnBars) do
            numPlayersOnBars = numPlayersOnBars + 1
        end
        local barsInDisplay = floor(weizPVP_CoreFrame:GetHeight() / rowHeight)
        local rawCalc = weizPVP_CoreFrame:GetHeight() / rowHeight
        if rawCalc - barsInDisplay >= .5 then
            barsInDisplay = barsInDisplay + 1
        end
        if barsInDisplay < 1 then
            barsInDisplay = 1
        end
        local minVerticalScroll = (numPlayersOnBars - barsInDisplay) * rowHeight
        if minVerticalScroll < 0 then
            minVerticalScroll = 0
        end
        local offset
        if value > 0 then
            offset = vertScroll - (rowHeight)
        else
            offset = vertScroll + (rowHeight)
        end
        if offset < 0 then
            weizPVP_CoreFrame.ScrollFrame:SetVerticalScroll(0)
        elseif offset > offsetMax then
            weizPVP_CoreFrame.ScrollFrame:SetVerticalScroll(offsetMax)
        elseif minVerticalScroll < offset then
            weizPVP_CoreFrame.ScrollFrame:SetVerticalScroll(minVerticalScroll)
        else
            weizPVP_CoreFrame.ScrollFrame:SetVerticalScroll(offset)
        end
    end
end

-- |> Core Bar
-----------------------------------------------------------
--> Init
local function CoreBarInit()
    --> CoreBar
    do
        weizPVP_CoreBar:RegisterForDrag("LeftButton")
        weizPVP_CoreBar:SetBackdrop(defaultBackdrop)
        weizPVP_CoreBar:SetBackdropColor(unpack(NS.Options.Frames.Header.BackgroundColor))
        weizPVP_CoreBar:SetBackdropBorderColor(0, 0, 0, 1)
        weizPVP_CoreBar:SetHeight(NS.Options.Frames.Header.Height)
        weizPVP_CoreBar:SetFrameLevel(8)

        --> Scripts
        function weizPVP_CoreBar:DragStart()
            if (not NS.Options.Window.Pinned) and (not NS.Options.Window.Locked) then
                weizPVP_CoreFrame:StartMoving()
                weizPVP_CoreFrame.moving = true
            elseif NS.Options.Window.Locked then
                CUI.SetIcon("lock")
                CUI.FlashTitleBar("lightGrey")
            else
                CUI.SetIcon("pin")
                CUI.FlashTitleBar("lightGrey")
            end
        end

        function weizPVP_CoreBar:DragStop()
            weizPVP_CoreFrame:StopMovingOrSizing()
            weizPVP_CoreFrame.moving = nil
            weizPVP_CoreFrame.resizing = nil
            NS.SaveCoreFramePosition()
            if self:IsMouseOver() == true then
                showHeaderToolTip()
            end
        end

        function weizPVP_CoreBar:MouseUp(button)
            if button == "LeftButton" then
                if self.moving then
                    self:StopMovingOrSizing()
                    self.moving = nil
                    NS.SaveCoreFramePosition()
                end
            end
            if self:IsMouseOver() and button == "RightButton" then
                if IsControlKeyDown() then
                    CUI.ToggleLock()
                    showHeaderToolTip()
                else
                    if not NS.Options.Window.Locked then
                        CUI.TogglePin()
                        showHeaderToolTip()
                    end
                end
            end
        end

        function weizPVP_CoreBar:MouseEnter()
            if not self.moving then
                showHeaderToolTip()
            end
        end
        function weizPVP_CoreBar:MouseLeave()
            if GameTooltip:GetOwner() == self then
                GameTooltip:Hide()
            end
        end
    end

    --: local reused vars
    local spacer = Round(NS.Options.Frames.Header.Height / 3)
    local spacerHalf = Round(NS.Options.Frames.Header.Height / 6)

    --> Status Icon
    do
        weizPVP_CoreBar.StatusIcon:ClearAllPoints()
        weizPVP_CoreBar.StatusIcon:SetPoint("TOPLEFT", weizPVP_CoreBar, "TOPLEFT", NS.Options.Frames.BorderSize * 4, 0)
        weizPVP_CoreBar.StatusIcon.AddonIcon:SetSize(NS.Options.Frames.Header.Height, NS.Options.Frames.Header.Height)
        weizPVP_CoreBar.StatusIcon.AddonIcon:SetSize(NS.Options.Frames.Header.Height, NS.Options.Frames.Header.Height)

        --: State Glyph
        weizPVP_CoreBar.StatusIcon.StateGlyph:SetHeight(NS.Options.Frames.Header.Height)
        weizPVP_CoreBar.StatusIcon.StateGlyph:SetWidth(NS.Options.Frames.Header.Height)
        weizPVP_CoreBar.StatusIcon.StateGlyphBG:SetHeight(NS.Options.Frames.Header.Height)
        weizPVP_CoreBar.StatusIcon.StateGlyphBG:SetWidth(NS.Options.Frames.Header.Height)
        weizPVP_CoreBar.StatusIcon.StateGlyphBG:ClearAllPoints()
        weizPVP_CoreBar.StatusIcon.StateGlyphBG:SetPoint(
            "CENTER",
            weizPVP_CoreBar.StatusIcon.StateGlyph,
            "CENTER",
            NS.Options.Frames.BorderSize * 1,
            NS.Options.Frames.BorderSize * -1
        )
        weizPVP_CoreBar.StatusIcon.StateGlyphBG:SetHeight(NS.Options.Frames.Header.Height)
        weizPVP_CoreBar.StatusIcon.StateGlyphBG:SetWidth(NS.Options.Frames.Header.Height)
    end

    --> Title (Nearby Count)
    weizPVP_CoreBar.Title:SetFont(
        SM:Fetch("font", NS.Options.Frames.Header.Font),
        NS.Options.Frames.Header.Height - Round(NS.Options.Frames.Header.Height / 4),
        "OUTLINE"
    )
    weizPVP_CoreBar.Separator:SetFont(SM:Fetch("font", NS.Options.Frames.Header.Font), NS.Options.Frames.Header.Height - spacer, "OUTLINE")
    weizPVP_CoreBar.Separator:SetText(":")
    weizPVP_CoreBar.Separator:SetPoint("TOP", weizPVP_CoreBar, "TOP", 0, 0)

    --> Expander Button
    do
        weizPVP_CoreBar.ExpanderButton:ClearAllPoints()
        weizPVP_CoreBar.ExpanderButton:SetPoint("RIGHT", weizPVP_CoreBar, "RIGHT", -spacer, 0)
        weizPVP_CoreBar.ExpanderButton:SetHeight(NS.Options.Frames.Header.Height - spacer)
        weizPVP_CoreBar.ExpanderButton:SetWidth(NS.Options.Frames.Header.Height - spacer)
        weizPVP_CoreBar.ExpanderButton:SetHitRectInsets(-(spacerHalf / 2), -spacer, -spacerHalf, -spacerHalf)
        weizPVP_CoreBar.ExpanderButton:GetNormalTexture():SetVertexColor(0.7, 0.7, 0.7)
        weizPVP_CoreBar.ExpanderButton:GetHighlightTexture():SetVertexColor(1, 1, 1)
        --: OnMouseDown
        weizPVP_CoreBar.ExpanderButton:SetScript(
            "OnMouseUp",
            function(self, button)
                if not InCombatLockdown() then
                    GameTooltip:SetOwner(weizPVP_CoreBar, "ANCHOR_TOPRIGHT")
                    if NS.Options.Window.Collapsed then --: collapse to expand
                        if button == "LeftButton" then
                            self.textureRotation = 0
                            self:StopAnimating()
                            self.FadeIn:Play()
                            CUI.ExpandFrame()
                            GameTooltip:SetText("Collapse")
                        end
                    else --: expand to collapse
                        if button == "LeftButton" then
                            self.textureRotation = 3.14159
                            self:StopAnimating()
                            self.FadeIn:Play()
                            CUI.CollapseFrame()
                            GameTooltip:SetText("Expand")
                        end
                    end
                    GameTooltip:Show()
                end
            end
        )
        --: OnEnter
        weizPVP_CoreBar.ExpanderButton:SetScript(
            "OnEnter",
            function()
                GameTooltip:SetOwner(weizPVP_CoreBar, "ANCHOR_TOPRIGHT")
                if NS.Options.Window.Collapsed then
                    GameTooltip:SetText("Expand")
                else
                    GameTooltip:SetText("Collapse")
                end
                GameTooltip:Show()
            end
        )
        --: OnLeave
        weizPVP_CoreBar.ExpanderButton:SetScript(
            "OnLeave",
            function()
                if GameTooltip:GetOwner() == weizPVP_CoreBar then
                    GameTooltip:Hide()
                else
                    GameTooltip:SetOwner(weizPVP_CoreBar, "ANCHOR_TOP")
                    GameTooltip:Hide()
                end
            end
        )
    end
    --> Clear List
    do
        weizPVP_CoreBar.ClearListButton:ClearAllPoints()
        weizPVP_CoreBar.ClearListButton:SetPoint("RIGHT", weizPVP_CoreBar.ExpanderButton, "LEFT", -spacerHalf, 0)
        weizPVP_CoreBar.ClearListButton:SetHeight(weizPVP_CoreBar.ExpanderButton:GetHeight())
        weizPVP_CoreBar.ClearListButton:SetWidth(weizPVP_CoreBar.ExpanderButton:GetWidth())
        weizPVP_CoreBar.ClearListButton:SetHitRectInsets(-spacerHalf, -(spacerHalf / 2), -spacerHalf, -spacerHalf)
        weizPVP_CoreBar.ClearListButton:GetNormalTexture():SetVertexColor(0.8, 0.5, 0) -- 0.9, 0.6, 0
        weizPVP_CoreBar.ClearListButton:GetHighlightTexture():SetVertexColor(1, 0.65, 0)
        weizPVP_CoreBar.ClearListButton:SetScript(
            "OnMouseDown",
            function(self, button)
                if button == "LeftButton" then
                    NS.ClearListData()
                    self.ClearAnim:Play()
                end
            end
        )
        weizPVP_CoreBar.ClearListButton:SetScript(
            "OnEnter",
            function()
                GameTooltip:SetOwner(weizPVP_CoreBar, "ANCHOR_TOPRIGHT")
                GameTooltip:SetText("Clear List")
                GameTooltip:Show()
            end
        )
        weizPVP_CoreBar.ClearListButton:SetScript(
            "OnLeave",
            function()
                GameTooltip:Hide()
            end
        )
    end
    --> Set CoreBar insets
    local insetValue = Round((weizPVP_CoreBar.ExpanderButton:GetWidth() * 2) + (spacerHalf * 3))
    insetValue = insetValue or 40
    weizPVP_CoreBar:SetHitRectInsets(0, insetValue, 0, 0)

    --: Status Highlight
    local highlightColor = "lightGrey"
    if NS.ColorsLUT[highlightColor] then
        weizPVP_CoreBar.StatusHeader.Highlight.grad:SetGradientAlpha(
            "HORIZONTAL",
            NS.ColorsLUT[highlightColor].R,
            NS.ColorsLUT[highlightColor].G,
            NS.ColorsLUT[highlightColor].B,
            0,
            NS.ColorsLUT[highlightColor].R,
            NS.ColorsLUT[highlightColor].G,
            NS.ColorsLUT[highlightColor].B,
            0.4
        )
    end
    weizPVP_CoreBar.StatusHeader.Highlight.grad:ClearAllPoints()
    weizPVP_CoreBar.StatusHeader.Highlight.grad:SetPoint("TOP", weizPVP_CoreBar, "TOP", 0, -1)
    weizPVP_CoreBar.StatusHeader.Highlight.grad:SetPoint("BOTTOM", weizPVP_CoreBar, "BOTTOM", 0, 1)
    weizPVP_CoreBar.StatusHeader.Highlight.grad:SetPoint("LEFT", weizPVP_CoreBar, "LEFT")
    weizPVP_CoreBar.StatusHeader.Highlight.grad:SetPoint("RIGHT", weizPVP_CoreBar.Separator, "CENTER")

    if NS.ColorsLUT[highlightColor] then
        weizPVP_CoreBar.StatusHeader.Highlight.grad2:SetGradientAlpha(
            "HORIZONTAL",
            NS.ColorsLUT[highlightColor].R,
            NS.ColorsLUT[highlightColor].G,
            NS.ColorsLUT[highlightColor].B,
            0.4,
            NS.ColorsLUT[highlightColor].R,
            NS.ColorsLUT[highlightColor].G,
            NS.ColorsLUT[highlightColor].B,
            0
        )
    end
    weizPVP_CoreBar.StatusHeader.Highlight.grad2:ClearAllPoints()
    weizPVP_CoreBar.StatusHeader.Highlight.grad2:SetPoint("TOP", weizPVP_CoreBar, "TOP", 0, -1)
    weizPVP_CoreBar.StatusHeader.Highlight.grad2:SetPoint("BOTTOM", weizPVP_CoreBar, "BOTTOM", 0, 1)
    weizPVP_CoreBar.StatusHeader.Highlight.grad2:SetPoint("LEFT", weizPVP_CoreBar.Separator, "CENTER")
    weizPVP_CoreBar.StatusHeader.Highlight.grad2:SetPoint("RIGHT", weizPVP_CoreBar, "CENTER")

    -- |> CORE FRAME
    weizPVP_CoreFrame:SetMinResize(140, NS.Options.Bars.RowHeight + NS.Options.Frames.Header.Height)
    weizPVP_CoreFrame:SetMaxResize(600, NS.Options.Frames.List.Height + NS.Options.Frames.Header.Height)
    -- weizPVP_CoreFrame:SetHitRectInsets(0, 0, NS.Options.Frames.Header.Height, 0)
    weizPVP_CoreFrame:RegisterForDrag("LeftButton")
    --> Scroll Frame
    do
        weizPVP_CoreFrame.ScrollFrame:SetBackdrop(defaultBackdrop)
        weizPVP_CoreFrame.ScrollFrame:SetBackdropColor(unpack(NS.Options.Frames.BackgroundColor))
        weizPVP_CoreFrame.ScrollFrame:SetBackdropBorderColor(0, 0, 0, 1)
        weizPVP_CoreFrame.ScrollFrame:SetPoint("TOP", weizPVP_CoreFrame, "TOP", 0, -NS.Options.Frames.Header.Height)
        --: ScrollFrame: MOUSEWHEEL
        weizPVP_CoreFrame.ScrollFrame:SetScript(
            "OnMouseWheel",
            function(_, value)
                AdustScrollOffset(value)
            end
        )
        --: ListFrame - (ScrollFrame's Scroll Child)
        weizPVP_CoreFrame.ScrollFrame.ListFrame:ClearAllPoints()
        weizPVP_CoreFrame.ScrollFrame.ListFrame:SetPoint("CENTER", weizPVP_CoreFrame.ScrollFrame, "CENTER")
        weizPVP_CoreFrame.ScrollFrame.ListFrame:SetPoint("TOP", weizPVP_CoreFrame, "TOP", 0, -NS.Options.Frames.Header.Height)
        weizPVP_CoreFrame.ScrollFrame.ListFrame:SetWidth(weizPVP_CoreFrame.ScrollFrame:GetWidth())
        weizPVP_CoreFrame.ScrollFrame.ListFrame:SetHeight(NS.Options.Frames.List.Height)

        --: Set ListFrame as scroll child to ScrollFrame
        weizPVP_CoreFrame.ScrollFrame:SetScrollChild(weizPVP_CoreFrame.ScrollFrame.ListFrame)
    end
    --> Resize BottomRight
    do
        weizPVP_CoreFrame.ResizeBottomRight:GetNormalTexture():SetVertexColor(1, 1, 1, 0.3)
        weizPVP_CoreFrame.ResizeBottomRight:GetHighlightTexture():SetVertexColor(1, 0.8, 0, 1)
        --: OnMouseDown
        weizPVP_CoreFrame.ResizeBottomRight:SetScript(
            "OnMouseDown",
            function(_, button)
                if (not NS.Options.Window.Locked) and button == "LeftButton" then
                    weizPVP_CoreFrame.resizing = true
                    weizPVP_CoreFrame:StartSizing("BOTTOMRIGHT")
                elseif NS.Options.Window.Locked == true then
                    NS.SetStatusMessage("Window is " .. sc.locked, true, 3)
                    CUI.SetIcon("lock")
                end
            end
        )
        --: OnMouseUp
        weizPVP_CoreFrame.ResizeBottomRight:SetScript(
            "OnMouseUp",
            function(_, button)
                if (button == "LeftButton") then
                    if not NS.Options.Window.Locked then
                        CUI.SnapHeightToBars()
                        if weizPVP_CoreFrame.resizing == true then
                            weizPVP_CoreFrame:StopMovingOrSizing()
                            weizPVP_CoreFrame.resizing = nil
                        end
                        NS.RefreshCurrentList()
                    end
                end
            end
        )
    end
    --> Resize BottomLeft
    do
        weizPVP_CoreFrame.ResizeBottomLeft:GetNormalTexture():SetVertexColor(1, 1, 1, 0.3)
        weizPVP_CoreFrame.ResizeBottomLeft:GetHighlightTexture():SetVertexColor(1, 0.8, 0, 1)
        weizPVP_CoreFrame.ResizeBottomLeft:GetNormalTexture():SetTexCoord(1, 0, 0, 1)
        weizPVP_CoreFrame.ResizeBottomLeft:GetHighlightTexture():SetTexCoord(1, 0, 0, 1)
        --: OnMouseDown
        weizPVP_CoreFrame.ResizeBottomLeft:SetScript(
            "OnMouseDown",
            function(_, button)
                if (not NS.Options.Window.Locked) and button == "LeftButton" then
                    weizPVP_CoreFrame.resizing = true
                    weizPVP_CoreFrame:StartSizing("BOTTOMLEFT")
                elseif NS.Options.Window.Locked == true then
                    NS.SetStatusMessage("Window is " .. sc.locked, true, 3)
                    CUI.SetIcon("lock")
                end
            end
        )
        --: OnMouseUp
        weizPVP_CoreFrame.ResizeBottomLeft:SetScript(
            "OnMouseUp",
            function(_, button)
                if (button == "LeftButton") then
                    if not NS.Options.Window.Locked then
                        CUI.SnapHeightToBars()
                        if weizPVP_CoreFrame.resizing == true then
                            weizPVP_CoreFrame:StopMovingOrSizing()
                            weizPVP_CoreFrame.resizing = nil
                        end
                        NS.RefreshCurrentList()
                    end
                end
            end
        )
    end

    --: AFTER SETTINGS
    weizPVP_CoreFrame:SetScale(ConvertPixelsToUI(1, UIParent:GetEffectiveScale()))
end

--|> Local Build Functions
-----------------------------------------------------------
--> Core Frame

--> Targeted Icon
local function BuildTargetedIcon()
    --: Outer Icon Frame
    weizPVP_CoreFrame.OuterIconFrame = weizPVP_CoreFrame.OuterIconFrame or CreateFrame("Frame", nil, weizPVP_CoreFrame)
    weizPVP_CoreFrame.OuterIconFrame:SetPoint("TOPLEFT", weizPVP_CoreFrame, "TOPLEFT", -NS.Options.Bars.RowHeight, 0)
    weizPVP_CoreFrame.OuterIconFrame:SetPoint("BOTTOMRIGHT", weizPVP_CoreFrame, "BOTTOMRIGHT", NS.Options.Bars.RowHeight, 0)
    weizPVP_CoreFrame.OuterIconFrame:SetClipsChildren(true)
    --: Targeted Icon
    weizPVP_CoreFrame.TargetedIcon = weizPVP_CoreFrame.TargetedIcon or weizPVP_CoreFrame.OuterIconFrame:CreateTexture(nil, "OVERLAY")
    weizPVP_CoreFrame.TargetedIcon:SetTexture("Interface\\Addons\\weizPVP\\Media\\Textures\\target.tga") --32x32 image
    weizPVP_CoreFrame.TargetedIcon:SetSize(
        Round(NS.Options.Bars.RowHeight - (NS.Options.Bars.RowHeight / 6)),
        Round(NS.Options.Bars.RowHeight - (NS.Options.Bars.RowHeight / 6))
    )
    weizPVP_CoreFrame.TargetedIcon:Hide()
end

--|> Initialize
-----------------------------------------------------------
local initializing = false
local initialized = false
function CUI.Initialize()
    if initializing == true then
        return
    end
    if initialized == true then
        weizPVP_CoreFrame:StopAnimating()
        weizPVP_CoreBar:StopAnimating()
    else
        localInit()
        initialized = true
    end

    initializing = true
    do
        CoreBarInit()
        BuildTargetedIcon()
        CUI.BuildBars()
        CUI.BuildStatusMessage()
        NS.SetCoreFramePosition()
        NS.SetWindowSettings()
        NS.ManageBarsDisplayed()
        NS.RefreshCurrentList()
        NS.UpdateNearbyCount()
        CUI.SnapHeightToBars()
        CUI.ChangeTargetIcon()
    end
    initializing = false
end

--|> Functions
-----------------------------------------------------------

--> Auto Vertical Scroll - for snapping height to bars
local function AutoVerticalScroll()
    if not InCombatLockdown() then
        local frameHeight = weizPVP_CoreFrame:GetHeight()
        local rowHeight = NS.Options.Bars.RowHeight + NS.Options.Bars.VerticalSpacing
        local rawCalc = frameHeight / rowHeight
        local barsInDisplay = floor(frameHeight / rowHeight)
        if rawCalc - barsInDisplay >= .5 then
            barsInDisplay = barsInDisplay + 1
        end
        if barsInDisplay < 1 then
            barsInDisplay = 1
        end
        local numPlayersOnBars = 0
        for _ in pairs(NS.PlayersOnBars) do
            numPlayersOnBars = numPlayersOnBars + 1
        end
        local minVerticalScroll = (numPlayersOnBars - barsInDisplay) * rowHeight
        if barsInDisplay > numPlayersOnBars then
            weizPVP_CoreFrame.ScrollFrame:SetVerticalScroll(0)
        elseif minVerticalScroll < weizPVP_CoreFrame.ScrollFrame:GetVerticalScroll() then
            weizPVP_CoreFrame.ScrollFrame:SetVerticalScroll(minVerticalScroll)
        end
    end
end

--> Snap to Bars
function CUI.SnapHeightToBars()
    if not InCombatLockdown() then
        local rowHeight = NS.Options.Bars.RowHeight + NS.Options.Bars.VerticalSpacing
        local barsInDisplay
        local frameHeight = weizPVP_CoreFrame:GetHeight()
        local rawCalc = frameHeight / rowHeight
        barsInDisplay = floor(frameHeight / rowHeight)
        if rawCalc - barsInDisplay >= .5 then
            barsInDisplay = barsInDisplay + 1
        end
        if barsInDisplay < 1 then
            barsInDisplay = 1
        end
        weizPVP_CoreFrame:SetHeight(barsInDisplay * rowHeight)
        AutoVerticalScroll()
        NS.SaveCoreFramePosition()
    end
end

--> Change Target Icon
local lastRowSelected = nil
function CUI.ChangeTargetIcon()
    if not weizPVP_CoreFrame then
        return
    end
    local GUID = UnitGUID("target")
    if NS.PlayersOnBars[GUID] then
        if weizPVP_CoreFrame:GetLeft() < 8 then
            weizPVP_CoreFrame.TargetedIcon:SetRotation(3.14159)
            weizPVP_CoreFrame.TargetedIcon:ClearAllPoints()
            weizPVP_CoreFrame.TargetedIcon:SetPoint("LEFT", CUI.Bar[NS.PlayersOnBars[GUID]], "RIGHT", -(NS.Options.Frames.BorderSize), 0)
        else
            weizPVP_CoreFrame.TargetedIcon:SetRotation(0)
            weizPVP_CoreFrame.TargetedIcon:ClearAllPoints()
            weizPVP_CoreFrame.TargetedIcon:SetPoint("RIGHT", CUI.Bar[NS.PlayersOnBars[GUID]], "LEFT", NS.Options.Frames.BorderSize, 0)
        end
        weizPVP_CoreFrame.TargetedIcon:Show()
        if lastRowSelected then
            CUI.Bar[lastRowSelected]:SetStatusBarTexture(SM:Fetch("statusbar", NS.Options.Bars.Texture))
        end
        CUI.Bar[NS.PlayersOnBars[GUID]]:SetStatusBarTexture(SM:Fetch("statusbar", NS.Options.Bars.BarSolid))
        lastRowSelected = NS.PlayersOnBars[GUID] or nil
    else
        weizPVP_CoreFrame.TargetedIcon:Hide()
        if lastRowSelected then
            CUI.Bar[lastRowSelected]:SetStatusBarTexture(SM:Fetch("statusbar", NS.Options.Bars.Texture))
        end
    end
end

--> Window: Set Window Settings
function NS.SetWindowSettings()
    if not InCombatLockdown() then
        if NS.Options.Window.Visible == true then
            weizPVP_CoreFrame:Show()
            if NS.Options.Window.Collapsed == false then
                NS.CoreUI.ExpandFrame()
            else
                NS.CoreUI.CollapseFrame()
            end
        else
            weizPVP_CoreFrame:Hide()
        end
    end
end

--> Set Status Bar Message
function NS.SetStatusMessage(StatusBarText, QuickMessage, Duration)
    if QuickMessage == true then -- Message will be temporary
        weizPVP_StatusMessage.Flash:Stop()
        weizPVP_StatusMessage.Clipped.Message:SetText(StatusBarText)
        weizPVP_StatusMessage.Clipped.SlideIn:SetEndDelay(Duration)
        weizPVP_StatusMessage.Clipped.FadeIn:SetEndDelay(Duration)
        weizPVP_StatusMessage.Flash:Play()
    else -- Static Message
        weizPVP_StatusMessage.Flash:Stop()
        weizPVP_StatusMessage.Clipped.Message:SetText(StatusBarText)
        weizPVP_StatusMessage.Clipped.SlideIn:SetEndDelay(1200)
        weizPVP_StatusMessage.Clipped.FadeIn:SetEndDelay(1200)
        weizPVP_StatusMessage.Flash:Play()
    end
end

--> Set Status Bar Message
local count
function NS.AutoResize()
    count = NS.NearbyCount or 0
    if NS.NearbyCount > NS.Options.Bars.MaxNumBars then
        count = NS.Options.Bars.MaxNumBars
    end
    if count == 0 and NS.Options.Window.Collapsed == false then
        CUI.CollapseFrame()
        count = 1
    elseif count > 0 and NS.Options.Window.Collapsed == true then
        CUI.ExpandFrame()
    end
    weizPVP_CoreFrame:SetHeight(count * (NS.Options.Bars.RowHeight + NS.Options.Bars.VerticalSpacing))
end

--> Set Icon
function CUI.SetIcon(icon)
    if not icon then
        return
    end
    if NS.IconLUT[icon] then
        weizPVP_CoreBar.StatusIcon.StateGlyph:SetText(NS.IconLUT[icon])
        weizPVP_CoreBar.StatusIcon.StateGlyphBG:SetText(NS.IconLUT[icon])
        weizPVP_CoreBar.StatusIconNotice:Stop()
        weizPVP_CoreBar.StatusIconNotice:Play()
    end
end

--> Toggle Pin
function CUI.TogglePin()
    if NS.Options.Window.Locked == true then
        CUI.SetIcon("lock")
        return
    end
    if NS.Options.Window.Pinned == true then
        CUI.SetIcon("unpin")
        NS.Options.Window.Pinned = false
        weizPVP_CoreBar:SetMovable(true)
    else
        CUI.SetIcon("pin")
        NS.Options.Window.Pinned = true
        weizPVP_CoreBar:SetMovable(false)
    end
end

--> Toggle Lock
function CUI.ToggleLock()
    if NS.Options.Window.Locked == true then
        CUI.SetIcon("unlock")
        NS.Options.Window.Locked = false
        weizPVP_CoreBar:SetMovable(true)
        weizPVP_CoreFrame.ResizeBottomLeft:Show()
        weizPVP_CoreFrame.ResizeBottomRight:Show()
    else
        CUI.SetIcon("lock")
        NS.Options.Window.Locked = true
        NS.Options.Window.Pinned = false
        weizPVP_CoreBar:SetMovable(false)
        weizPVP_CoreFrame.ResizeBottomLeft:Hide()
        weizPVP_CoreFrame.ResizeBottomRight:Hide()
    end
end

--> Collapse Frame
function CUI.CollapseFrame()
    weizPVP_CoreFrame:SetClampedToScreen(false)
    weizPVP_CoreFrame.ScrollFrame:Hide()
    weizPVP_CoreFrame.ResizeBottomLeft:Hide()
    weizPVP_CoreFrame.ResizeBottomRight:Hide()
    NS.Options.Window.Collapsed = true
    weizPVP_CoreFrame.TargetedIcon:Hide()
end

--> Expand Frame
function CUI.ExpandFrame()
    weizPVP_CoreFrame:SetClampedToScreen(true)
    weizPVP_CoreFrame.ScrollFrame:Show()
    weizPVP_CoreFrame.ResizeBottomLeft:Show()
    weizPVP_CoreFrame.ResizeBottomRight:Show()
    NS.Options.Window.Collapsed = false
    CUI.ChangeTargetIcon()
end

--> Combat Start
function CUI.CombatStart()
    CUI.SetStatusIconColor(NS.ColorsLUT["InCombat"].R, NS.ColorsLUT["InCombat"].G, NS.ColorsLUT["InCombat"].B)
    NS.SetStatusMessage(NS.ColorsLUT["InCombat"].sStart .. "IN COMBAT|r", true, 3)
    -- CUI.FlashTitleBar("InCombat")
end
--> Combat End
function CUI.CombatEnd()
    CUI.SetStatusIconColor(NS.ColorsLUT["OutOfCombat"].R, NS.ColorsLUT["OutOfCombat"].G, NS.ColorsLUT["OutOfCombat"].B)
    C_Timer_After(
        3,
        function()
            if not InCombatLockdown() then
                CUI.SetStatusIconColor(NS.ColorsLUT["addon"].R, NS.ColorsLUT["addon"].G, NS.ColorsLUT["addon"].B)
            end
        end
    )
    NS.SetStatusMessage(NS.ColorsLUT["OutOfCombat"].sStart .. "OUT OF COMBAT|r", true, 2.7)
end

--> Set Status Icon Color
function CUI.SetStatusIconColor(r, g, b)
    weizPVP_CoreBar.StatusIcon.AddonIconHighlight:SetVertexColor(r, g, b)
end

--> Flash Title Bar
function CUI.FlashTitleBar(color)
    if NS.ColorsLUT[color] then
        weizPVP_CoreBar.StatusHeader.Highlight.grad:SetGradientAlpha(
            "HORIZONTAL",
            NS.ColorsLUT[color].R,
            NS.ColorsLUT[color].G,
            NS.ColorsLUT[color].B,
            0,
            NS.ColorsLUT[color].R,
            NS.ColorsLUT[color].G,
            NS.ColorsLUT[color].B,
            0.4
        )
        weizPVP_CoreBar.StatusHeader.Highlight.grad2:SetGradientAlpha(
            "HORIZONTAL",
            NS.ColorsLUT[color].R,
            NS.ColorsLUT[color].G,
            NS.ColorsLUT[color].B,
            0.4,
            NS.ColorsLUT[color].R,
            NS.ColorsLUT[color].G,
            NS.ColorsLUT[color].B,
            0
        )
        if weizPVP_CoreBar.HeaderFlash:IsPlaying() then
            weizPVP_CoreBar.HeaderFlash:Restart()
        else
            weizPVP_CoreBar.HeaderFlash:Play()
        end
    else
        if weizPVP_CoreBar.HeaderFlash:IsPlaying() then
            weizPVP_CoreBar.HeaderFlash:Restart()
        else
            weizPVP_CoreBar.HeaderFlash:Play()
        end
    end
end
