-------------------------------------------------------------------------------
--|> Player Bars
--: Manages the display of the bars and their elements representing detected players
-------------------------------------------------------------------------------
local _, NS = ...
-- local UIFrameFadeIn = UIFrameFadeIn
-- local UIFrameFadeOut = UIFrameFadeOut
-- local InCombatLockdown = InCombatLockdown
-- local GetTime = GetTime

--> Core UI
local CUI = NS.CoreUI

--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

local sc = {}
local defaultBackdrop = {}
local function localInit()
    sc = {
        white8x8 = "Interface/BUTTONS/WHITE8X8",
        pinned = "|cFF" .. NS.ColorsLUT["Pinned"].hex .. "PINNED|r",
        unpinned = "|cFF" .. NS.ColorsLUT["Unpinned"].hex .. "UNPINNED|r",
        locked = "|cFF" .. NS.ColorsLUT["Locked"].hex .. "LOCKED|r",
        unlocked = "|cFF" .. NS.ColorsLUT["Unlocked"].hex .. "UNLOCKED|r"
    }

    defaultBackdrop = {
        bgFile = sc.white8x8,
        edgeFile = sc.white8x8,
        edgeSize = NS.Options.Frames.BorderSize
    }
end

--> Build Status Bar
local inDuration = 0.2
local outDuration = 0.3
function CUI.BuildStatusMessage()
    localInit()
    --> Status Bar
    weizPVP_StatusMessage:ClearAllPoints()
    weizPVP_StatusMessage:SetParent(weizPVP_CoreBar)
    weizPVP_StatusMessage:SetFrameStrata("MEDIUM")
    weizPVP_StatusMessage:SetFrameLevel(0)
    weizPVP_StatusMessage:SetPoint("BOTTOMLEFT", weizPVP_CoreBar, "TOPLEFT", 0, -1)
    weizPVP_StatusMessage:SetPoint("BOTTOMRIGHT", weizPVP_CoreBar, "TOPRIGHT", 0, -1)
    weizPVP_StatusMessage:SetClipsChildren(true)
    weizPVP_StatusMessage:SetHeight(NS.Options.Frames.StatusPopUp.Height + 2)

    -- --> Status Bar - Clipped
    weizPVP_StatusMessage.Clipped:ClearAllPoints()
    weizPVP_StatusMessage.Clipped:SetPoint("TOPLEFT", weizPVP_StatusMessage, "BOTTOMLEFT", 0, NS.Options.Frames.BorderSize)
    weizPVP_StatusMessage.Clipped:SetPoint("TOPRIGHT", weizPVP_StatusMessage, "BOTTOMRIGHT", 0, -NS.Options.Frames.BorderSize)
    weizPVP_StatusMessage.Clipped:SetHeight(NS.Options.Frames.StatusPopUp.Height + NS.Options.Frames.BorderSize)
    weizPVP_StatusMessage.Clipped:SetBackdrop(defaultBackdrop)
    weizPVP_StatusMessage.Clipped:SetBackdropColor(unpack(NS.Options.Frames.StatusPopUp.BackgroundColor))
    weizPVP_StatusMessage.Clipped:SetBackdropBorderColor(unpack(NS.Options.Frames.StatusPopUp.BorderColor))

    --> Text
    weizPVP_StatusMessage.Clipped.Message:SetFont(
        SM:Fetch("font", NS.Options.Frames.StatusPopUp.Font),
        Round(NS.Options.Frames.StatusPopUp.Height - (NS.Options.Frames.StatusPopUp.Height / 4)),
        NS.Options.Frames.StatusPopUp.FontOutline
    )
    weizPVP_StatusMessage.Clipped.Message:SetText("message")
    weizPVP_StatusMessage.Clipped.Message:SetShadowColor(0, 0, 0, 0.6)
    weizPVP_StatusMessage.Clipped.Message:SetShadowOffset(1, 0)

    --> Animations
    --: Flash Status
    -- weizPVP_StatusMessage.Flash = weizPVP_StatusMessage.Flash or weizPVP_StatusMessage:CreateAnimationGroup()
    weizPVP_StatusMessage.Clipped.FadeIn = weizPVP_StatusMessage.Clipped.FadeIn or weizPVP_StatusMessage.Flash:CreateAnimation("Alpha")
    weizPVP_StatusMessage.Clipped.FadeIn:SetDuration(inDuration)
    weizPVP_StatusMessage.Clipped.FadeIn:SetFromAlpha(0)
    weizPVP_StatusMessage.Clipped.FadeIn:SetToAlpha(1)
    weizPVP_StatusMessage.Clipped.FadeIn:SetEndDelay(2.5) -- Set the number of seconds the animation delays after finishing.
    weizPVP_StatusMessage.Clipped.FadeIn:SetOrder(1)
    weizPVP_StatusMessage.Clipped.SlideIn = weizPVP_StatusMessage.Clipped.SlideIn or weizPVP_StatusMessage.Flash:CreateAnimation("Translation")
    weizPVP_StatusMessage.Clipped.SlideIn:SetDuration(inDuration)
    weizPVP_StatusMessage.Clipped.SlideIn:SetOffset(0, NS.Options.Frames.StatusPopUp.Height + NS.Options.Frames.BorderSize)
    weizPVP_StatusMessage.Clipped.SlideIn:SetEndDelay(2.5) -- Set the number of seconds the animation delays after finishing.
    weizPVP_StatusMessage.Clipped.SlideIn:SetOrder(1)

    --> Slide Out
    weizPVP_StatusMessage.Clipped.FadeOut = weizPVP_StatusMessage.Clipped.FadeOut or weizPVP_StatusMessage.Flash:CreateAnimation("Alpha")
    weizPVP_StatusMessage.Clipped.FadeOut:SetDuration(outDuration)
    weizPVP_StatusMessage.Clipped.FadeOut:SetFromAlpha(1)
    weizPVP_StatusMessage.Clipped.FadeOut:SetToAlpha(0)
    weizPVP_StatusMessage.Clipped.FadeOut:SetOrder(2)
    weizPVP_StatusMessage.Clipped.SlideOut =
        weizPVP_StatusMessage.Clipped.SlideOut or weizPVP_StatusMessage.Flash:CreateAnimation("Translation")
    weizPVP_StatusMessage.Clipped.SlideOut:SetDuration(outDuration)
    weizPVP_StatusMessage.Clipped.SlideOut:SetOffset(0, -1 * select(2, weizPVP_StatusMessage.Clipped.SlideOut:GetOffset()))
    weizPVP_StatusMessage.Clipped.SlideOut:SetOrder(2)
end
