-----------------------------------------------------------------------
--|> ZONE DETECTION
-----------------------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace
--> UpValues
local select = select
local wipe = wipe
local C_Timer_After = C_Timer.After
local InCombatLockdown = InCombatLockdown
local IsInInstance = IsInInstance
local GetTime = GetTime
local GetZoneText = GetZoneText

--|> ZONES AND INSTANCES
-------------- ----------------------------------------------------------------------------
NS.Zone = {}
NS.Zone.instance = ""
NS.Zone.pvpType = ""
local ZoneTimerStartTime = GetTime()
local ValidBG
local ValidWorldZone

--> Check for Valid (PVP) Instance
local function CheckValidInstance()
    if NS.Zone.instance == "pvp" and NS.Options.Addon.EnabledInBattlegrounds == true then
        return true
    end
    if NS.Zone.instance == "arena" and NS.Options.Addon.EnabledInArena == true then
        return true
    end
    return false
end

--> Check for Valid World Zone
local function CheckValidWorldZone()
    --> Checking for Valid WPVP Zone
    if NS.Zone.instance == "none" then
        return true
    else
        return false
    end
end

--> Get PVP Zone
function NS.GetPVPZone()
    if ZoneTimerStartTime + 1 <= GetTime() then
        ZoneTimerStartTime = GetTime()
        if GetZoneText() == "" then
            C_Timer_After(0.5, NS.GetPVPZone)
            return
        end
        NS.Zone.pvpType = select(1, GetZonePVPInfo())
        NS.Zone.instance = select(2, IsInInstance())
        ValidWorldZone = CheckValidWorldZone()
        ValidBG = CheckValidInstance()
        if not InCombatLockdown() then
            if ValidWorldZone or ValidBG then
                weizPVP:OnEnable()
                weizPVP_CoreFrame:Show()
                if NS.Options.Window.Collapsed == false then
                  weizPVP_CoreFrame:Show()
                end
            else
                weizPVP:OnDisable()
            end
        end
    else
        return
    end
end

--> EVENT: Entering Battleground Instance
function NS.PlayerEnteringBattlegroundEvent()
    C_Timer_After(1, NS.GetPVPZone)
    wipe(NS.CurrentNameplates)
end

--> EVENT: ZoneChangedNewAreaEvent
function NS.ZoneChangedNewAreaEvent()
    C_Timer_After(1, NS.GetPVPZone)
end

--> EVENT: Loading Screen Leave
function NS.LoadingScreenLeave()
    NS.CoreUI.Initialize()
end

--> EVENT: Player Entering World
function NS.PlayerEnteringWorldEvent()
    if NS.addonInitializing then -- check if this is the first 'entering world' run since init was ran
        NS.PrintAddonMessage(NS.Constants.AddonString .. " |cffcccccc" .. GetAddOnMetadata("weizPVP", "Version") .. "|r : |cff37ff37Initialized!|r")
        NS.addonInitializing = nil
    end
    --> ENABLES
    NS.EnableLDB()
    wipe(NS.CurrentNameplates) -- 'nameplate removed' event wont always fire, manually wipe
    NS.Crosshair.NewTarget() -- check for target
    C_Timer_After(1, NS.GetPVPZone)
    if NS.databaseReset then -- database update check for pre-1.9.0
        C_Timer_After(
            1,
            function()
                NS.databaseReset = nil
                StaticPopup_Show("WEIZPVP_UPGRADE_DB_RESET")
                NS.PrintAddonMessage("|cffFFA200Saved options and data have been reset!|r")
                NS.PrintAddonMessage(
                    "The database reset was required to resolve potential issues with the new KOS features and player tracking, especially for players that play on multiple realms."
                )
                NS.PrintAddonMessage("Database updates in the future will migrate all your data and settings; No more resets will be needed.")
            end
        )
    end
    NS.Phasing.FirstRun = true
    C_Timer_After(1, NS.Phasing.Enabled)
    C_Timer_After(
        1,
        function()
            NS.Phasing.FirstRun = nil
        end
    )
    NS.CoreUI.Initialize()
end
